@@ -115,6 +115,12 @@ public:
    */
   bool force = false;
 
+  /** @brief Database-only retirement
+   *
+   * The default is @c false.
+   */
+  bool forgetOnly = false;
+
   /** @brief Log summary verbosity */
   LogVerbosity logVerbosity = Failed;
 
@@ -140,10 +146,10 @@ public:
 
 private:
   /** @brief Display help message and terminate */
-  void help();
+  [[noreturn]] void help();
 
   /** @brief Display version string and terminate */
-  void version();
+  [[noreturn]] void version();
 };
 
 /** @brief Program command line */
